-- Thok
freeslot("MT_MOMTHOK", "S_MOMTHOK_EXIST")

-- Ha ha yes spinning emeralds
freeslot("MT_EMERALDSPIN")

mobjinfo[MT_EMERALDSPIN] = {
	health = 1000,
	flags = MF_NOCLIP|MF_NOCLIPTHING|MF_NOCLIPHEIGHT|MF_NOGRAVITY,
	spawnstate = S_INVISIBLE,
}

addHook("MobjThinker", function(mo)
	-- If the timer is nil, set it to 0
	if not mo.timer then mo.timer = 0 end
	-- Gradually lower opacity and increase scale
	mo.frame = $ | (FF_TRANS10 * (mo.timer/3))
	mo.scale = $ + (mo.spawnscale/TICRATE*4)
	
	-- Increase timer
	mo.timer = mo.timer + 1
	
	-- If timer reaches 35, kill the mobj
	-- Wouldn't want it to stay there forever, would you?
	if mo.timer > 35 then
		P_KillMobj(mo, nil, nil)
	end
end, MT_MOMTHOK)

local function UM_SpawnThok(p)
	-- Spawn the thok object
	local mo = P_SpawnMobj(p.mo.x, p.mo.y, p.mo.z, MT_MOMTHOK)
	
	-- Set color and scale accordingly
	mo.color = p.mo.color
	mo.scale = p.mo.scale + (p.mo.scale / 4)
	
	-- Set spawn scale for the scale increase
	mo.spawnscale = mo.scale
	
	mo.angle = p.mo.angle + ANGLE_90
	return mo
end
rawset(_G, "UM_SpawnThok", UM_SpawnThok)
local function UM_SpawnLand(mo)
	local e = 0
	if mo.eflags & MFE_VERTICALFLIP then
		e = mo.height
	end
	UM_SpawnCircle(MT_SPINDUST, mo.x, mo.y, mo.z + e, 7*FRACUNIT, 6, mo.scale, mo.momx, mo.momy, mo.pmomz, mo.eflags & MFE_VERTICALFLIP)
end
rawset(_G, "UM_SpawnLand", UM_SpawnLand)
local function UM_SpawnCircle(type, x, y, z, spd, amount, scale, carryx, carryy, carryz, flip)
	local step = FixedDiv(360*FRACUNIT, amount*FRACUNIT)
	for i=0,amount do
		local mo = P_SpawnMobj(x, y, z, type)
		if flip then mo.flags2 = $ | MF2_OBJECTFLIP end
		mo.scale = scale
		local a = FixedAngle(step*i)
		mo.angle = a
		mo.momx = FixedMul(FixedMul(sin(a), spd), scale) + (carryx or 0)
		mo.momy = FixedMul(FixedMul(cos(a), spd), scale) + (carryy or 0)
		mo.momz = carryz or 0
	end
end
rawset(_G, "UM_SpawnCircle", UM_SpawnCircle)

-- Speed lines
freeslot( "MT_SPEEDLINE", "S_SPEEDLINE_EXIST")
addHook("MobjThinker", function(mo)
	-- If the timer is nil, set it to 0
	if not mo.timer then mo.timer = 0 end
	-- Gradually lower opacity depending on timer
	mo.frame = $ | (FF_TRANS20 + (FF_TRANS10 * mo.timer))
	
	-- Increase timer
	mo.timer = mo.timer + 1
	
	-- If timer reaches 17, kill the mobj
	-- Wouldn't want it to stay there forever, would you?
	if mo.timer > 17 then
		P_KillMobj(mo, nil, nil)
	end
end, MT_SPEEDLINE)
addHook("MobjThinker", function(mo)
	if mo.target and mo.target.valid then P_TeleportMove(mo, mo.target.x, mo.target.y, mo.target.z) end
	if not mo.emeralds then
		mo.spawnscale = mo.scale
		mo.timer = 0
		mo.emeralds = {
			P_SpawnMobjFromMobj(mo, 0, 0, 0, MT_EMERALD1),
			P_SpawnMobjFromMobj(mo, 0, 0, 0, MT_EMERALD2),
			P_SpawnMobjFromMobj(mo, 0, 0, 0, MT_EMERALD3),
			P_SpawnMobjFromMobj(mo, 0, 0, 0, MT_EMERALD4),
			P_SpawnMobjFromMobj(mo, 0, 0, 0, MT_EMERALD5),
			P_SpawnMobjFromMobj(mo, 0, 0, 0, MT_EMERALD6),
			P_SpawnMobjFromMobj(mo, 0, 0, 0, MT_EMERALD7)
		}
	end

	if #mo.emeralds then
		local i = 0
		local step = FRACUNIT*360 / #mo.emeralds
		local dist = mo.scale * 100
		for _,v in pairs(mo.emeralds) do
			if leveltime % 2 == 0 then
				P_SpawnGhostMobj(v)
			end
			if (mo.timer % #mo.emeralds == i) then
				P_SpawnMobjFromMobj(v, 0, 0, 0, MT_SUPERSPARK)
			end
			v.flags = $ | MF_NOCLIPTHING | MF_NOCLIP | MF_NOCLIPHEIGHT
			local ang = FixedAngle(i * step) + mo.angle
			P_TeleportMove(v, mo.x + FixedMul(sin(ang), dist), mo.y + FixedMul(cos(ang), dist), mo.z)
			i = $ + 1
		end
	end
	mo.angle = $ + ANGLE_180/TICRATE*2
	mo.timer = $ + 1
	if mo.timer >= TICRATE/4 then 
		mo.scale = $ - (mo.spawnscale/TICRATE + mo.scale/TICRATE/4)
		if mo.timer == TICRATE then
			for _,v in pairs(mo.emeralds) do
				local g = P_SpawnGhostMobj(v)
				g.frame = $ & (~FF_TRANSMASK)
				P_RemoveMobj(v)
			end
			mo.emeralds = {}
			mo.color = SKINCOLOR_WHITE
			mo.blendmode = AST_ADD
			mo.scale = mo.spawnscale
		end
		if mo.timer > TICRATE then

			if mo.timer > TICRATE + 30 then
				P_RemoveMobj(mo)
			end
		end
	end
end, MT_EMERALDSPIN)
local function UM_SpawnSpeedLine(p, zofs, ofs, color)
	-- Spawn the speed line object
	local mo = P_SpawnMobj(p.mo.x, p.mo.y, p.mo.z, MT_SPEEDLINE)
	
	-- Set color and scale accordingly
	mo.color = color or p.mo.color
	mo.scale = p.mo.scale
	mo.momx = FixedMul(p.mo.momx / 2, P_RandomRange(FRACUNIT, FRACUNIT*2))
	mo.momy = FixedMul(p.mo.momy / 2, P_RandomRange(FRACUNIT, FRACUNIT*2))
	mo.momz = FixedMul(p.mo.momz / 2, P_RandomRange(FRACUNIT, FRACUNIT*2))
	local total = abs(p.mo.momx) + abs(p.mo.momy) + abs(p.mo.momz)
	local z = FixedDiv(p.mo.momz, total)
	mo.rollangle = FixedAngle(FixedMul(z, -90*FRACUNIT))
	mo.angle = R_PointToAngle2(p.mo.x, p.mo.y, p.mo.x + p.mo.momx, p.mo.y + p.mo.momy)
	
	zofs = FixedMul(zofs, mo.scale)
	ofs = FixedMul(ofs, mo.scale)
	if p.mo.eflags & MFE_VERTICALFLIP then
		zofs = -zofs
	end
	
	P_TeleportMove(mo, mo.x + P_ReturnThrustX(mo, mo.angle - ANGLE_90, ofs), mo.y + P_ReturnThrustY(mo, mo.angle - ANGLE_90, ofs), mo.z + zofs)
	return mo
end
rawset(_G, "UM_SpawnSpeedLine", UM_SpawnSpeedLine)